@@ -20,44 +20,29 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-#
-# Standard python modules
-#
 import os
 import re
 import sys
 import tempfile
 import signal
-#
+
+import apt_pkg
+import apt
+
+# wajig modules
+import changes
+import perform
+import util
+
 # When writing to a pipe where there is no reader (e.g., when
 # output is directed to head or to less and the user exists from less
 # before reading all output) the SIGPIPE signal is generated. Capture
 # the signal and hadle it with the default handler.
-#
 signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
-#
-# APT module
-#
-# import apt
-import apt_pkg
-
-#
-# Wajig modules
-#
-import changes
-import perform
-
-#
-# Global Variables
-#
-available_file = changes.get_available_filename()
-previous_file  = changes.get_previous_filename()
-
-#
-# Interface Variables
-#
 verbose = 0
+available_file = changes.available_file
+previous_file  = changes.previous_file
 
 
 def set_verbosity_level(new_level):
@@ -70,31 +55,30 @@ def commify(strnum):
     if len(strnum) <= 3:
         return strnum
     else:
-        pre = strnum[0:-3]
+        pre = strnum[:-3]
         post = strnum[-3:]
         pre = commify(pre)
         return pre + "," + post
-#
-# SUPPORT
-#
+
+
 def ping_host(hostname):
-    #
+    "Check if host is reachable."
+
     # Check if fping is installed.
-    #
     if perform.execute("fping localhost 2>/dev/null >/dev/null",
                        display=False) != 0:
         print "JIG Warning: fping was not found. " +\
               "Consider installing the package fping.\n"
-    #
+
     # Check if we can talk to the HOST
-    #
     elif perform.execute("fping " + hostname + " 2>/dev/null >/dev/null",
                          display=False) != 0:
-        print "JIG Warning: Could not contact the Debian server at\n" +\
-              "             " + hostname + """
+        print "JIG Warning: Could not contact the Debian server at " + hostname\
+        + """
              Perhaps it is down or you are not connected to the network.
-             JIG will continue to try to get the information required.
-"""
+             JIG will continue to try to get the information required."""
+    else:
+        return True  # host found
 
 
 def get_available(command="dumpavail"):
@@ -117,18 +101,15 @@ def get_available(command="dumpavail"):
     # append /var/lib/dpkg/status to be more comprehensive? Would
     # repeats be an issue? Closes Bug#432266
 
-    tmpcache = tempfile.mktemp()
-    command = "apt-cache dumpavail > " + tmpcache
-    perform.execute(command)
-    # 090501 Begin
-    command = "cat /var/lib/dpkg/status >> " + tmpcache
-    perform.execute(command)
-    # 090501 End
+    tmpcache = tempfile.mkstemp()[1]
+    perform.execute("apt-cache dumpavail > " + tmpcache)
+    perform.execute("cat /var/lib/dpkg/status >> " + tmpcache)  # 090501 fix
     avail = apt_pkg.TagFile(file(tmpcache))
     if os.path.exists(tmpcache):
         os.remove(tmpcache)
     return avail
 
+
 def do_dependents(package):
     command = "apt-cache showpkg " + package +\
               " | awk \'BEGIN{found=0}/^Reverse Depends:/{found=1;next}/^[^ ]/{found=0}found==1{print}{next}\' " +\
@@ -139,9 +120,8 @@ def do_dependents(package):
     for pkg in packages:
         pkgs.append(pkg.strip())
     avail = get_available()
-    #
+
     # Check for information in the Available list
-    #
     for section in avail:
         nam = section.get("Package")
         if (nam in pkgs):
@@ -183,12 +163,7 @@ def do_dependents(package):
 #
 #------------------------------------------------------------------------
 def do_describe(packages):
-    """Print a description of each package.
-
-    Arguments:
-    packages    List of packages (or deb files) to describe
-
-    Returns:"""
+    "Display a description of a package(s)."
 
     #
     # From where do we get information about the packages?
@@ -211,12 +186,6 @@ def do_describe(packages):
     # the dpkg available list /var/lib/dpkg/available for now and
     # make sure UPDATE uses "dselect update"
     #
-    # Compare also with gnome-tasksel which does the following:
-    #
-    #  apt-get update
-    #  apt-cache dumpavail > /tmp/avail
-    #  dpkg --update-avail /tmp/avail
-    #
     # avail  = apt_pkg.TagFile(open("/var/cache/apt/available","r"));
     # avail  = apt_pkg.TagFile(open("/var/cache/apt/pkgcache.bin","r"));
     # avail  = apt_pkg.TagFile(open("/var/lib/dpkg/available","r"))
@@ -231,11 +200,11 @@ def do_describe(packages):
     # First, extract the package names and the package files
     # and handle separately.
     #
-    package_files = filter(lambda x: x[-4:] == '.deb', packages)
-    package_names = filter(lambda x: x[-4:] != '.deb', packages)
+    package_files = [pkg for pkg in packages if pkg.endswith(".deb")]
+    package_names = [pkg for pkg in packages if not pkg.endswith(".deb")]
     if package_files:
-        for f in package_files:
-            perform.execute("dpkg-deb --info %s" % f)
+        for package_file in package_files:
+            perform.execute("dpkg-deb --info " + package_file)
             print "="*72
             sys.stdout.flush()
 
@@ -260,14 +229,9 @@ def do_describe(packages):
     else:
         return
     avail = get_available()
+    describe_list = dict()
 
-    #
-    # Record the descriptions
-    #
-    describe_list = {}
-    #
     # Check for information in the Available list
-    #
     for section in avail:
         if (section.get("Package") in packages):
             package_name = section.get("Package")
@@ -283,20 +247,21 @@ def do_describe(packages):
 #             describe_list[pkgname] = cache[pkgname].description
     # End Bug Fix for Bug#366678
 
-    #
     # Print out the one line descriptions. Should it be sorted?
     # If not sorted it should be same order as on the command line.
-    #
     pkgs = describe_list.keys()
     pkgs.sort()
-    #
-    # Print the description depending on level of detail requested
-    # through the value of "verbose"
-    #
-    if len(pkgs) == 0:
-        print "No packages found from those known to be available or installed"
+
+    # hack - ensures that we reach aptitude's 'virtual pkg' message
+    cache = apt.Cache()
+    if cache.get_providing_packages("motho"):
+        pkgs = [1]
+
+    # Print the description, depending on level of detail (verbose).
+    if len(pkgs) == 0 and verbose < 2:  # 'verbose < 2' is part of above hack
+        print "No packages found from those known to be available/installed."
     elif verbose == 0:
-        print "%-24s %s" % ("Package", "Description")
+        print "{0:24} {1}".format("Package", "Description")
         print "="*24 + "-" + "="*51
 
 #
@@ -328,13 +293,12 @@ def do_describe(packages):
         for pkg in pkgs:
             print pkg + ": " + describe_list[pkg].capitalize() + "\n"
     else:
-        #
         # TODO is there a way of doing this using apt_pkg easily?
-        # Otherwise "apt-cache show" seems okay if a little slower.
+        # Use "aptitude show" for now.
         #
 # Comment out for fix for Bug#366678
-        package_names = perform.concat(packages)
-        command = "apt-cache show " + package_names
+        package_names = util.concat(packages)
+        command = "aptitude show " + package_names
         perform.execute(command)
 # Bug#366678 fix from mvo - part 3 - not working yet???
 #   for pkgname in filter(lambda pkgname: cache.has_key(pkgname), packages):
@@ -353,11 +317,7 @@ def do_describe(packages):
 #
 #------------------------------------------------------------------------
 def do_describe_new(install=False):
-    """Report on packages that are newly available.
-
-    Arguments:
-
-    Returns:"""
+    "Report on packages that are newly available."
 
     #
     # Describe each new package.
@@ -377,14 +337,9 @@ def do_describe_new(install=False):
 #
 #------------------------------------------------------------------------
 def do_download(packages):
-    """Download packages.
+    "Download packages without installing them."
 
-    Arguments:
-    packages    List of packages to be downloaded
-
-    Returns:"""
-
-    command = "apt-get --download-only install " + perform.concat(packages)
+    command = "apt-get --download-only install " + util.concat(packages)
     perform.execute(command, root=1)
 
 #------------------------------------------------------------------------
@@ -398,11 +353,7 @@ def do_force(packages):
     This is useful when there is a conflict of the same file from
     multiple packages or when a dependency is not installed for
     whatever reason.
-
-    Arguments:
-    packages    The packages to be installed (either names or path to deb)
-
-    Returns:"""
+    """
     #
     # The basic function is to force install the package using dpkg.
     #
@@ -461,12 +412,7 @@ def do_force(packages):
 #
 #------------------------------------------------------------------------
 def do_hold(packages):
-    """Place packages on hold (they will not be upgraded).
-
-    Arguments:
-    packages    List of packages.
-
-    Returns:"""
+    "Place packages on hold (so they will not be upgraded)."
 
     for p in packages:
         # The dpkg needs sudo but not the echo.
@@ -482,14 +428,8 @@ def do_hold(packages):
 # INSTALL
 #
 #------------------------------------------------------------------------
-def do_install(packages, noauth=""):
-    """Install packages.
-
-    Arguments:
-    packages    List of packages to be installed or path to deb file
-    noauth      A string which might be --allow-unathenticated
-
-    Returns:"""
+def do_install(packages, yes=False, noauth=False):
+    "Install packages."
 
     #
     # Currently we use the first argument to determine the type of all
@@ -505,29 +445,26 @@ def do_install(packages, noauth=""):
     # Currently only a single URL is allowed. Should this be generalised?
     #
 
-    #
-    # If reading from standard input generate the list of packages.
-    #
+    # reading packages from stdin
     if len(packages) == 1 and packages[0] == "-":
         stripped = [x.strip() for x in sys.stdin.readlines()]
         joined = str.join(stripped)
         packages = joined.split()
-    #
-    # If reading packages from file generate list of packages.
-    #
+
+    # rading packages from a file
     elif len(packages) == 2 and packages[0] == "-f":
         stripped = [x.strip() for x in open(packages[1]).readlines()]
         joined = str.join(stripped)
         packages = str.split(joined)
-    #
-    # Check if a specific web location was specified.
-    #
-    if re.match("(http|ftp)://", packages[0]):
+
+    # check if a specific web location was specified
+    if re.match("(http|ftp)://", packages[0]) \
+       and util.requires_package("wget", "/usr/bin/wget"):
         if len(packages) > 1:
             print "wajig: Error: install URL allows only one URL, not " +\
                   str(len(packages))
             sys.exit(1)
-        tmpdeb = tempfile.mktemp() + ".deb"
+        tmpdeb = tempfile.mkstemp()[1] + ".deb"
         command = "wget --output-document=" + tmpdeb + " " + packages[0]
         if not perform.execute(command):
             command = "dpkg --install " + tmpdeb
@@ -537,10 +474,8 @@ def do_install(packages, noauth=""):
         else:
             print "Wajig: The location " + packages[0] +\
                   " was not found. Check and try again."
-            return
-    #
-    # Check if a specific .DEB file was specified
-    #
+
+    # check if a specific .DEB file was specified
     elif re.match(".*\.deb$", packages[0]):
         command = "dpkg -i"
         for pkg in packages:
@@ -573,97 +508,68 @@ def do_install(packages, noauth=""):
 #         print "HI"
 #         (packages[0], release) = re.compile(r'/\+').split(packages[0])
 #       command = "apt-get --target-release %s install %s" %\
-#                   (release, perform.concat(packages))
+#                   (release, util.concat(packages))
 #       perform.execute(command, root=1)
     else:
-        command = "apt-get %s install " % noauth + perform.concat(packages)
-        perform.execute(command, root=1)
+        perform.execute("apt-get {0} {1} install {2}".format(yes, noauth,
+                         util.concat(packages)), root=True)
 
 #------------------------------------------------------------------------
 #
 # INSTALL SUGGEST
 #
 #------------------------------------------------------------------------
-def do_install_suggest(packages, type, noauth=""):
-    """Install packages suggested by the listed packages.
+def do_install_suggest(packages, deptype, noauth=""):
+    "Install packages suggested by the listed packages."
 
-    Arguments:
-    packages    List of packages
-    noauth      A string which might be --allow-unathenticated
-
-    Returns:"""
-
-    #
-    # If reading from stanard input generate the list of packages
-    #
+    # If reading from standard input, generate the list of packages.
     if len(packages) == 1 and packages[0] == "-":
         stripped = [x.strip() for x in sys.stdin.readlines()]
         joined = str.join(stripped)
         packages = joined.split()
 
-    #
     # For each package obtain the list of suggested packages
-    #
     suggest_list = ""
 
-    ## Obtain the details of each available package.
-
+    # Obtain the details of each available package.
     avail = get_available()
-    #
+
     # Check for information in the Available list
-    #
     for section in avail:
-        if (section.get("Package") in packages):
+        if section.get("Package") in packages:
             sug = section.get("Suggests")
             rec = section.get("Recommends")
-            if type == "Suggests" or type == "Both":
+            if deptype in ("Suggests", "Both"):
                 if sug:
-                    suggest_list = suggest_list+" "+sug
-            if type == "Recommends" or type == "Both":
+                    suggest_list = "{0} {1}".format(suggest_list, sug)
+            if deptype in ("Recommends", "Both"):
                 if rec:
-                    suggest_list = suggest_list+" "+rec
-    #
+                    suggest_list = "{0} {1}".format(suggest_list, rec)
+
     # Remove version information
-    #
     suggest_list = re.sub('\([^)]*\)', '', suggest_list)
-    #
+
     # Remove the commas from the list
-    #
     suggest_list = re.sub(',', '', suggest_list)
-    #
+
     # Deal with alternatives.
     # For now simply ignore all alternatives.
     # Should prompt user to select one?
-    #
     suggest_list = re.sub(' *[a-z0-9-]* *\| *[a-z0-9-]* *', ' ', suggest_list)
-    #
+
     # Remove duplicates
-    #
     suggest_list = suggest_list + " "
     for i in suggest_list.split():
         suggest_list = i + " " + re.sub(" " + i + " ", ' ', suggest_list)
-    #
-    # For the list of supplied packages and the list of suggested packages
-    # do an install
-    #
-    # I used to ask but get apt-get to do the asking.
-    #
-    #if len(string.strip(suggest_list)) > 0:
-    #  print "Installing: " + suggest_list
-    #  cont = raw_input('Continue (Y/n)? ')
-    #else:
-    #    cont = ""
-    #if cont == "" or cont == "Y" or cont == "y" or cont == "yes":
 
-    # 100224 Bug #402598 Use aptitude -r to get the recursive
-    # recommends installed, rather than going just one level.
-
-    if type == "Recommends":
+    # Use aptitude -r to get the recursive recommends installed, rather
+    # than going just one level - #402598.
+    if deptype == "Recommends":
         command = "aptitude %s -r --show-deps --show-versions install " % \
-            noauth + perform.concat(packages) + suggest_list
+            noauth + util.concat(packages) + suggest_list
     else:
         command = "apt-get %s --show-upgraded install " % noauth \
-            + perform.concat(packages) + suggest_list
+            + util.concat(packages) + suggest_list
     perform.execute(command, root=1)
 
 #-----------------------------------------------------------------------
@@ -693,7 +599,7 @@ def do_listsection(pattern):
 #
 #------------------------------------------------------------------------
 def do_listinstalled(pattern):
-    """Print list of installed packages."""
+    "Display a list of installed packages."
     command = "dpkg --get-selections | awk '$2 ~/^install$/ {print $1}'"
     if len(pattern) == 1:
         command = command + " | grep -- " + pattern[0] + " | sort -k 1b,1"
@@ -705,15 +611,10 @@ def do_listinstalled(pattern):
 #
 #------------------------------------------------------------------------
 def do_listnames(pattern, pipe=False):
-    """Print list of known package names.
+    "Print list of known package names."
 
-    Arguments:
-
-    Returns:"""
-    #
     # If user can't access /etc/apt/sources.list then must do this with
     # sudo or else most packages will not be found.
-    #
     needsudo = not os.access("/etc/apt/sources.list", os.R_OK)
     if len(pattern) == 0:
         command = "apt-cache pkgnames | sort -k 1b,1"
@@ -756,11 +657,7 @@ def do_listscripts(pkg):
 #
 #------------------------------------------------------------------------
 def do_new():
-    """Report on packages that are newly available.
-
-    Arguments:
-
-    Returns:"""
+    "Report on packages that are newly available."
 
     print "%-24s %s" % ("Package", "Available")
     print "="*24 + "-" + "="*16
@@ -773,145 +670,79 @@ def do_new():
         print "%-24s %s" % (new_pkgs[i],
             changes.get_available_version(new_pkgs[i]))
 
-#------------------------------------------------------------------------
-#
-# NEWS
-#
-#------------------------------------------------------------------------
-def map_sources(packages):
-    """Return Source package name for each package in PACKAGES.
-
-    Note that the source could include a version number.
-    Only replace with source if package exists (Bug#295455).
-"""
-    tagfile = apt_pkg.TagFile(open("/var/lib/dpkg/available", "r"))
-    sources = []
-    for section in tagfile:
-        # May want to look to terminate as soon as all packages
-        # have been found - to be more efficient
-        if section.get("Package") in packages:
-            if section.get("Source"):
-                #pkgname = section.get("Source").split()[0]
-                pkgname = section.get("Source")
-                sources += [[pkgname, section.get("Package")]]
-            else:
-                sources += [[section.get("Package"), section.get("Package")]]
-    return sources
-
-
-def do_news(packages):
-    """List latest news for each package.
-
-    Arguments:
-    packages    List the latest news for each.
-
-    Returns: News items
-    """
-    ping_host("packages.debian.org")
-    results = tempfile.mktemp()
-    packages = map_sources(packages)
-    if not packages:
-        # list the original package name (maybe it is a source package)
-        # packages = map(lambda(x): [x,x], orig)
-        print "No package of that name found - perhaps it's a source package."
-    for pk in packages:
-        p = pk[0]
-        #
-        # Print a header.
-        #
-        print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
-        #
-        #
-        #
-        command  = "wget --timeout=60 --output-document=-"
-        command += " http://packages.debian.org/changelog:" + p.split()[0]
-        command += " 2> /dev/null "
-        #
-        # Strip out only the first entry in the changelog.
-        #
-        command += "| awk 'NR==1{print;next} /^[^ ]/{exit}{print;next}' "
-        perform.execute(command)
-        if os.path.exists(results):
-            os.remove(results)
 
 #------------------------------------------------------------------------
 #
 # CHANGELOG
-# Updated 2004-06-05 21:05:39 graham
-# Updated 2004-10-19 07:00:08 graham Change host name
 #
 #------------------------------------------------------------------------
-def do_changelog(packages):
-    """List latest changelog for each package.
+def local_changelog(package, tmp):
+    "Retrieve Debian changelog from local installation."
 
-    Arguments:
-    packages    List the latest changelog for each.
+    changelog = "/usr/share/doc/" + package + "/changelog.Debian.gz"
+    changelog_native = "/usr/share/doc/" + package + "/changelog.gz"
+    if os.path.exists(changelog):
+        return "zcat {0} >> {1}".format(changelog, tmp)
+    elif os.path.exists(changelog_native):
+        return "zcat {0} >> {1}".format(changelog_native, tmp)
+    else:
+        print "Package", package, "is likely broken (changelog not found)!"
 
-    Returns: Changelogs
+
+def do_changelog(package, pager):
+    """Display Debian changelog.
+    network on:
+         changelog - if there's newer entries, display them
+      -v changelog - if there's newer entries, display them, and proceed to
+                     display complete local changelog
+      -x changelog - same as "-v changelog", but use a pager
+    network off:
+         changelog - if there's newer entries, mention failure to retrieve
+      -v changelog - if there's newer entries, mention failure to retrieve, and
+                     proceed to display complete local changelog
+      -x changelog - same as "-v changelog", but use a pager
     """
-    ping_host("packages.debian.org")
-    packages = map_sources(packages)
-    if not packages:
-        # list the original package name (maybe it is a source package)
-        # packages = map(lambda(x): [x,x], orig)
-        print "No package of that name found - perhaps it's a source package."
-    for pk in packages:
 
-        # map_sources was changed (gjw 17 Feb 2005) to return pairs
-        # (source package name, original package name) so that we can
-        # track the original package name in case the source package
-        # is not an installable package. Fixes Bug #295455.  (gjw 23
-        # Apr 2006) With the change to using the changelog:pkg syntax,
-        # we don't any longer really need this.
+    changelog = "{0:=^72}\n".format(" {0} ".format(package))  # header
+    if pager:
+        pipe_cmd = "/usr/bin/sensible-pager "
+    else:
+        pipe_cmd = ""
 
-        p = pk[0]
-
-        # Print a header.
-
-        print "="*((70-len(p))/2) + " " + p + " " + "="*((70-len(p))/2)
-        sys.stdout.flush()
-
-        # Some source pakages have a different version which will be
-        # included in the actual packaname string: e.g. "bash
-        # (2.05b-2-15)".  We handle this case by using this version
-        # number rather than the package's installed version number.
-
-        pkg = p.split()
-        if len(pkg) > 1:
-            pkg = pkg[0]
+    pkg = apt.Cache()[package]
+    changelog += pkg.get_changelog()
+    help_message = "\nTo display the local changelog, run:\n" \
+                   "wajig --pager changelog " + package
+    if "Failed to download the list of changes" in changelog:
+        if not verbose:
+            changelog += help_message
         else:
-            pkg = pkg[0]
+            changelog += "\n"
+    elif changelog.endswith("The list of changes is not available"):
+        changelog += ".\nYou are likely running the latest version."
+        if not verbose:
+            changelog += help_message
+    if not verbose:
+        print changelog
+    else:
+        tmp = tempfile.mkstemp()[1]
+        with open(tmp, "w") as f:
+            if pkg.is_installed:
+                changelog += "\n{0:=^72}\n".format(" local changelog ")
+            changelog = changelog.encode("utf8")
+            f.write(changelog)
+        if pkg.is_installed:
+            cmd = local_changelog(package, tmp)
+            if not cmd:
+                return
+            perform.execute(cmd)
+        if pipe_cmd:
+            perform.execute(pipe_cmd + tmp)
+        else:
+            with open(tmp) as f:
+                for line in f:
+                    sys.stdout.write(line)
 
-        # Find pool file name and get latest version number.
-        # Previously I relied on changelogs.debian.net, but it seems
-        # that the symlink from, e.g.,
-        # http://changelogs.debian.net/metacity to the actual
-        # changelog file does not get updated very quickly.  So move
-        # to pointing the to latest version's actual changelog on
-        # packages.debian.org. (gjw 30 Dec 2004).
-
-        # Extra argument added 17 Feb for fix for Bug#295455. The orig
-        # package name is used to get version if source pkg is not an
-        # installable package.
-
-        # (gjw 23 Apr 2006) Move to using changelog:pkg
-        # syntax. Hopefully this takes care of the complications.
-        #
-        # (pool, version) = pool_version(pkg, o)
-        # changelog_path = re.sub(r'_[^_]*\.deb$', '/changelog', pool)
-
-        # Generate the command to retireve the changelog.
-
-        command = "wget --timeout=60 --output-document=-"
-        # (gjw 30 Dec 2004) command += " http://changelogs.debian.net/"
-        command += " http://packages.debian.org/"
-        #command += "changelogs/" + p[0] + "/"
-        #command += "%s/%s_%s/changelog" % (pkg, pkg, ver)
-        # (gjw 30 Dec 2004) command += "%s" % (pkg)
-        # (gjw 23 Apr 2006) Try changelog: command += "changelogs/%s" % (changelog_path)
-        command += "changelog:" + pkg
-        command += " 2> /dev/null"
-        perform.execute(command)
 
 #------------------------------------------------------------------------
 #
@@ -919,11 +750,7 @@ def do_changelog(packages):
 #
 #------------------------------------------------------------------------
 def do_newupgrades(install=False):
-    """Report on packages that are newly upgraded.
-
-    Arguments:
-
-    Returns:"""
+    "Display packages that are newly upgraded."
 
     #
     # Load the dictionaries from file then list each one and it's version
@@ -943,43 +770,17 @@ def do_newupgrades(install=False):
             print "="*74
             do_install(new_upgrades)
 
-# 100519 No longer used - user apt-get's --auto-remove instead
-
-def do_removedepend(pkg, purge=False):
-    print "Checking dependencies. Please wait..."
-    toremove = pkg
-    #
-    # Obtain list of dependencies.
-    #
-    dependencies = changes.get_dependencies(pkg)
-    #
-    # For each dependency, find dependents that are installed.
-    # If only one is this pkg, then add it to the removal list.
-    #
-    if dependencies:
-        for p in dependencies:
-            # Check if the supplied pkg is the only dependee
-            if changes.get_dependees(p) == [pkg]:
-                toremove += " " + p
-    #
-    # Now do the actual removal
-    #
-    print "Removing:", toremove
-    if purge:
-        perform.execute("apt-get remove --purge " + toremove, root=1)
-    else:
-        perform.execute("apt-get remove " + toremove, root=1)
 
 ########################################################################
 # SIZE
 #
-def do_size(packages, size):
-    """Print sizes for pkg in list PACAKAGES with size greater than SIZE."""
+def do_size(packages, size=0):
+    "Print sizes for pkg in list PACAKAGES with size greater than SIZE."
     #
     # Work with the list of installed packages
     # (I think status has more than installed?)
     #
-    status = apt_pkg.TagFile(open("/var/lib/dpkg/status", "r"));
+    status = apt_pkg.TagFile(open("/var/lib/dpkg/status", "r"))
     #
     # Initialise the sizes dictionary.
     #
@@ -1026,11 +827,10 @@ def do_status(packages, snapshot=False):
     Arguments:
     packages    List the version of installed packages
     snapshot    Whether a snapshot is required (affects output format)
-
-    Returns:"""
+    """
 
     if not snapshot:
-        print "%-23s %-15s %-15s %-15s %s" %\
+        print "%-23s %-15s %-15s %-15s %s" % \
               ("Package", "Installed", "Previous", "Now", "State")
         print "="*23 + "-" + "="*15 + "-" + "="*15 + "-" + "="*15 + "-" + "="*5
         sys.stdout.flush()
@@ -1045,7 +845,7 @@ def do_status(packages, snapshot=False):
     #
     # Generate a temporary file of installed packages.
     #
-    ifile = tempfile.mktemp()
+    ifile = tempfile.mkstemp()[1]
     #
     # Using langC=TRUE here makes it work for other LANG, e.g.,
     # LANG=ru_RU.koi8r. Seems that the sorting is the key problem. To
@@ -1118,11 +918,7 @@ def do_status(packages, snapshot=False):
 #
 #------------------------------------------------------------------------
 def do_toupgrade():
-    """List packages with Available version more recent than Installed.
-
-    Arguments:
-
-    Returns:"""
+    "List packages with Available version more recent than Installed."
 
     # A simple way of doing this is to just list packages in the installed
     # list and the available list which have different versions.
@@ -1151,17 +947,12 @@ def do_toupgrade():
 #
 #------------------------------------------------------------------------
 def do_unhold(packages):
-    """Remove packages from hold (they will again be upgraded).
+    "Remove packages from hold (they will again be upgraded)."
 
-    Arguments:
-    packages   List of packages.
-
-    Returns:"""
-
-    for p in packages:
+    for package in packages:
         # The dpkg needs sudo but not the echo.
         # Do all of it as root then.
-        command = "echo \"" + p + " install\" | dpkg --set-selections"
+        command = "echo \"" + package + " install\" | dpkg --set-selections"
         perform.execute(command, root=1)
     print "The following packages are still on hold:"
     perform.execute("dpkg --get-selections | egrep 'hold$' | cut -f1")
@@ -1172,18 +963,12 @@ def do_unhold(packages):
 #
 #------------------------------------------------------------------------
 def do_update(query=0, quiet=False):
-    """Perform.Execute an update of the available packages
-
-    Arguments:
-    query       If non-zero then ask user whether this should be done.
-
-    Returns:
-    Return status of the command"""
+    "Perform an update of the available packages."
 
     if verbose > 0:
         print "The Packages files listing the available packages is being \
                updated."
-    if query > 0:
+    if query > 0:  # ask user whether this should be done.
         versionfile = open("/etc/debian_version")
         debian_version = versionfile.read().split()[0]
         if debian_version == "testing/unstable":
@@ -1243,40 +1028,6 @@ likely that your Packages files are out of date. Doing an UPDATE is optional
 
         exit(1)
 
-#------------------------------------------------------------------------
-#
-# WHATIS
-#
-# No longer used! Use describe instead.
-#
-#------------------------------------------------------------------------
-def do_whatis0(packages):
-    """Look for a one-line description of the given packages.
-
-    Arguments:
-    packages    List of packages to get descriptions for
-
-    Returns:"""
-
-    command = "fping packages.debian.org 2>&1 >/dev/null"
-    if perform.execute(command) == 0:
-        for i in packages:
-            results = tempfile.mktemp()
-            command = "wget --output-document=" + results +\
-                      " http://packages.debian.org/cgi-bin" +\
-                      "/search_packages.pl\?keywords=" + i +\
-                      "\&searchon=names\&subword=1\&" +\
-                      "version=unstable\&release=all 2> /dev/null"
-            perform.execute(command)
-            command = "cat " + results + " | " +\
-                "egrep '<TD><B><A HREF|<TD COLSPAN=2>' | " +\
-                "perl -p -e 's|<[^>]*>||g;s|^    ||g;" +\
-                "s|^&nbsp; |    |;s|&nbsp;||g;s|&quot;|\"|g;'"
-            perform.execute(command)
-            if os.path.exists(results):
-                os.remove(results)
-    else:
-        print "Perhaps the server is temporarily down."
 
 #------------------------------------------------------------------------
 #
@@ -1291,10 +1042,9 @@ def do_whichpkg(filename):
     packages containing a file with FILENAME in its name. The results
     are printed to standard output.
 
-    Arguments:
+    Argument:
     filename    Pattern to find
-
-    Returns:"""
+    """
 
     packages_host = "packages.debian.org"
     ping_host(packages_host)
@@ -1325,7 +1075,7 @@ def do_whichpkg(filename):
     #
     print "-"*77
     print "AVAILABLE"
-    results = tempfile.mktemp()
+    results = tempfile.mkstemp()[1]
     filename = filename.replace('+', '%2B')
 
     # 100117 Bug#565666 Update to newer search syntax. TODO We should
@@ -1414,32 +1164,24 @@ def do_whichpkg(filename):
 #
 #------------------------------------------------------------------------
 def do_findpkg(pkg):
-    """Look for a particular pkg at apt-get.org.
+    "Look for a particular pkg at apt-get.org."
 
-    Arguments:
-    pkg         Package to find
-
-    Returns:"""
     ping_host("www.apt-get.org")
-    #
+
     # Print out a suitable heading
-    #
-    print "Lines suitable for /etc/apt/sources.list"
-    print ""
+    print "Lines suitable for /etc/apt/sources.list\n"
     sys.stdout.flush()
-    #
+
     # Obtain the information from the Apt-Get server
-    #
-    results = tempfile.mktemp()
+    results = tempfile.mkstemp()[1]
     command = "wget --timeout=60 --output-document=" + results +\
               " http://www.apt-get.org/" +\
               "search.php\?query=" + pkg +\
               "\&submit=\&arch%5B%5D=i386\&arch%5B%5D=all " +\
               "2> /dev/null"
     perform.execute(command)
-    #
+
     # A single page of output
-    #
     command = "cat " + results + " | " +\
               "egrep '(^deb|sites and .*packages)' | " +\
               "perl -p -e 's|<[^>]*>||g;s|<[^>]*$||g;s|^[^<]*>||g;'" +\
@@ -1461,13 +1203,8 @@ def do_recdownload(packages):
     #FIXME: This has problems with virtual packages, FIX THEM!!!
 
     """Download packages and all dependencies recursively.
-
-        Arguments:
-        packages       List of packages to download recursively
-
-        Returns:
-
-        Author: Juanjo Alvarez <juanjux@yahoo.es>"""
+    Author: Juanjo Alvarez <juanjux@yahoo.es>
+    """
 
     def get_deps(package):
         tagfile = apt_pkg.TagFile(open("/var/lib/dpkg/available", "r"))
@@ -1493,12 +1230,12 @@ def do_recdownload(packages):
 
     packageNames = []
     dontDownloadList = []
-    for p in packages[:]:
+    for package in packages[:]:
         # Ignore packages with a "-" at the end so the user can workaround some
         # dependencies problems (usually in unstable)
-        if p[len(p) - 1:] == "-":
-            dontDownloadList.append(p[:-1])
-            packages.remove(p)
+        if package[len(package) - 1:] == "-":
+            dontDownloadList.append(package[:-1])
+            packages.remove(package)
             continue
 
     print "Calculating all dependencies..."
@@ -1519,7 +1256,7 @@ def do_recdownload(packages):
     print "\n"
 
     command = "apt-get --download-only --reinstall -u install " \
-    + perform.concat(packageNames)
+    + util.concat(packageNames)
     perform.execute(command, root=1)
 
 
@@ -1528,13 +1265,12 @@ def versions(packages):
         command = "apt-show-versions"
     else:
         command = ""
-    for p in packages:
-        command += "apt-show-versions " + p + "; "
+    for package in packages:
+        command += "apt-show-versions " + package + "; "
     perform.execute(command)
 
 
-def main():
-    do_size([], 100)
-
-if __name__ == '__main__':
-    main()
+def rbuilddep(package):
+    cmd = "grep-available -sPackage -FBuild-Depends,Build-Depends-Indep " + \
+          package + " /var/lib/apt/lists/*Sources"
+    os.system(cmd)
