@@ -1,4 +1,3 @@
-#
 # Gnome Glade toolkit utilities
 #
 # Copyright (c) Graham Williams
@@ -20,7 +19,7 @@
 # Authors: Graham Williams borrowing from gnomeglade.py in meld.
 
 """Utility classes for working with glade files.
- 
+
 """
 import os
 import sys
@@ -39,10 +38,8 @@ except ImportError, e:
 #
 def appdir(path):
     return os.path.join(os.path.dirname(sys.argv[0]), path)
-########################################################################
-#
-# Base
-#
+
+
 class Base:
     """Base class for all glade objects.
 
@@ -68,9 +65,9 @@ class Base:
         # self.xml.signal_autoconnect(locals())
         #
         handlers = {}
-        for h in filter(lambda x:x.startswith("on_"), dir(self.__class__)):
+        for h in filter(lambda x: x.startswith("on_"), dir(self.__class__)):
             handlers[h] = getattr(self, h)
-        self.xml.signal_autoconnect( handlers )
+        self.xml.signal_autoconnect(handlers)
         self.widget = getattr(self, root)
         self.widget.set_data("pyobject", self)
         #
@@ -82,7 +79,7 @@ class Base:
         """Allow glade widgets to be accessed as self.widgetname.
         """
         widget = self.xml.get_widget(key)
-        if widget: # cache lookups
+        if widget:  # cache lookups
             setattr(self, key, widget)
             return widget
         raise AttributeError(key)
@@ -91,7 +88,7 @@ class Base:
         """Handle all the events currently in the main queue and return.
         """
         while gtk.events_pending():
-            gtk.main_iteration();
+            gtk.main_iteration()
 
     def _map_widgets_into_lists(self, widgetnames):
         """Put sequentially numbered widgets into lists.
@@ -105,7 +102,7 @@ class Base:
             list = getattr(self, item)
             i = 0
             while 1:
-                key = "%s%i"%(item,i)
+                key = "%s%i" % (item,i)
                 try:
                     val = getattr(self, key)
                 except AttributeError:
@@ -123,7 +120,7 @@ class Component(gtk.Widget, Base):
 
     def __init__(self, file, root):
         """Create from node 'root' in a specified file"""
-        Base.__init__(self,file,root)
+        Base.__init__(self, file, root)
 
 
 ########################################################################
@@ -150,17 +147,14 @@ class GnomeApp(gnome.ui.App, Base):
         """
         gtk.main_quit()
 
-########################################################################
-#
-# Dialog
-#
+
 class Dialog(gtk.Dialog, Base):
     """A convenience base class for dialogs created in glade"""
 
     def __init__(self, file, root):
         """Create from node 'root' in a specified file"""
         gtk.Dialog.__init__(self)
-        Base.__init__(self,file,root)
+        Base.__init__(self, file, root)
 
 ########################################################################
 #
@@ -171,37 +165,40 @@ class Dialog(gtk.Dialog, Base):
 class InfoDialog:
 
     def __init__(self, parent, message):
-        dlg=gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
+        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_INFO,
                               gtk.BUTTONS_OK,
                               message)
         dlg.run()
         dlg.destroy()
 
+
 class WarningDialog:
 
     def __init__(self, parent, message):
-        dlg=gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
+        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_WARNING,
                               gtk.BUTTONS_OK,
                               message)
         dlg.run()
         dlg.destroy()
 
+
 class ErrorDialog:
 
     def __init__(self, parent, message):
-        dlg=gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
+        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_ERROR,
                               gtk.BUTTONS_OK,
                               message)
         dlg.run()
         dlg.destroy()
 
+
 class QuestionDialog:
 
     def __init__(self, parent, message):
-        dlg=gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
+        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
                                    gtk.MESSAGE_QUESTION,
                                    gtk.BUTTONS_YES_NO,
                                    message)
@@ -212,10 +209,7 @@ class QuestionDialog:
     def get_answer(self):
         return self.answer
 
-########################################################################
-#
-# load_pixbuf
-#
+
 def load_pixbuf(fname, size=0):
     """Load an image from a file as a pixbuf, with optional resizing.
     """
