@@ -24,7 +24,6 @@
 #------------------------------------------------------------------------
 import os
 import getpass
-pause = False
 
 setroot = "/bin/su"
 #
@@ -73,9 +72,9 @@ def set_quiet(check=True):
         quiet = ""
 
 
-def set_teaching_level(new_level):
+def set_teaching():
     global teaching
-    teaching = new_level
+    teaching = True
 
 
 def set_simulate(new_level):
@@ -83,13 +82,6 @@ def set_simulate(new_level):
     simulate = new_level
 
 
-def concat(args):
-    result = ""
-    for a in args:
-        result = "{0}'{1}' ".format(result, a)
-    return result
-
-
 def execute(command, root=False, noquiet=False, display=True, pipe=False,
             langC=False, test=False):
     """Ask the operating system to perform a command.
